from __future__ import annotations

from .eng.AskUbuntuDupQuestions import *
from .eng.MindSmallReranking import *
from .eng.SciDocsReranking import *
from .eng.StackOverflowDupQuestions import *
from .eng.WebLINXCandidatesReranking import *
from .fra.AlloprofReranking import *
from .fra.SyntecReranking import *
from .multilingual.MIRACLReranking import *
from .multilingual.WikipediaRerankingMultilingual import *
from .rus.RuBQReranking import *
from .zho.CMTEBReranking import *
